close all
clear all

%loads reward structure
environment
n_choices=length(reward_structure(1,:));

for iter=1:1000
    
    %Montecarlo method
    parameters(iter,1)=rand(1)+0.01; %random search of parameters, in the interval 0.01<x<1.01
    parameters(iter,2)=rand(1);      %random search of parameters, in the interval 0<x<1
    
    %learning rate for RL agent
    alpha=parameters(iter,1);
    %standard deviation for Bayesian agent
    sigma1 =parameters(iter,2);

    %temperature for softmax selection function (set as hyper parameter)
    tau  =0.1;
    
    %determine the assumed distribution for the possible events
    values1 = normpdf(1:n_choices,1,sigma1);
    m1=values1(1);
    s1=values1(2);
    %a matrix here is mostly useful in the case of multi-option choices (more than 2) 
    B_probN=[0 1
             1 0]*s1 + eye(n_choices)*m1; %for events that confirm the belief
    B_probB=[0 1
             1 0]*m1 + eye(n_choices)*s1; %for events the conflict with the belief

    for triali=1:length(reward_structure(:,1))
        
        %benchmark: random actions
        random_agent_action=randi(2);    
        random_agent_reward=rew_assign(reward_structure(triali,random_agent_action));
        rand_agent(triali,:)=[random_agent_action random_agent_reward];
        
        %RL agent
        if triali==1
            Q_values(triali,:)=[0.5 0.5];
        elseif triali>=2
            %Q value update (Rescorla Wagner): this updates the value
            %associated with the action selected in the previous trial. The remaining
            %action is reset to zero in this basic version of the algorithm.
            Q_values(triali,RL_agent_action)=Q_values(triali-1,RL_agent_action) + alpha*(RL_agent_reward-Q_values(triali-1,RL_agent_action));
        end
        %select an action based on Q value, using a softmax
        RL_behaviour=softmaxselection(Q_values(triali,:), tau);
        RL_agent_action=RL_behaviour{1};
        RL_agent_reward=rew_assign(reward_structure(triali,RL_agent_action));
        RL_agent(triali,:)=[RL_agent_action RL_agent_reward];    

        %Bayesian agent
        if triali==1
            %initial distribution of probabilities
            Bayes_p(triali,:)=ones(1,n_choices)/n_choices;
        elseif triali>=2
            %update of the distribution of probabilities
            if Bayes_agent_action==1
                if Bayes_agent_reward==100 ||  Bayes_agent_reward==-50
                    den=Bayes_p(triali-1,1)*B_probN(1,1)+Bayes_p(triali-1,2)*B_probN(1,2);
                    Bayes_p(triali,:)=(B_probN(Bayes_agent_action,:).*Bayes_p(triali-1,:))/den;
                elseif  Bayes_agent_reward==-100 ||  Bayes_agent_reward==50
                    den=Bayes_p(triali-1,1)*B_probN(2,1)+Bayes_p(triali-1,2)*B_probN(2,2);
                    Bayes_p(triali,:)=(B_probB(Bayes_agent_action,:).*Bayes_p(triali-1,:))/den;
                end
            elseif Bayes_agent_action==2
                if Bayes_agent_reward==100 ||  Bayes_agent_reward==-50
                    den=Bayes_p(triali-1,1)*B_probN(2,1)+Bayes_p(triali-1,2)*B_probN(2,2);
                    Bayes_p(triali,:)=(B_probN(Bayes_agent_action,:).*Bayes_p(triali-1,:))/den;
                elseif  Bayes_agent_reward==-100 ||  Bayes_agent_reward==50
                    den=Bayes_p(triali-1,1)*B_probN(1,1)+Bayes_p(triali-1,2)*B_probN(1,2);
                    Bayes_p(triali,:)=(B_probB(Bayes_agent_action,:).*Bayes_p(triali-1,:))/den;
                end
            end
        end

        %select an action, based on probability distribution
        rb=rand(1);
        for ind=1:length(Bayes_p(triali,:))
            if rb<=Bayes_p(triali,ind)
                Bayes_agent_action=ind;
                break
            else
                rb=rb-Bayes_p(triali,ind);
            end
        end
        %assign a reward
        Bayes_agent_reward=rew_assign(reward_structure(triali,Bayes_agent_action));
        Bayes_agent(triali,:)=[Bayes_agent_action Bayes_agent_reward];

        %if one of the options reaches values close to zero, updates for 
        %that option stall: this below is a simple way to prevent this
        Bayes_p(triali,:)=max(0.1, Bayes_p(triali,:));

    end
    
    %best parameter to maximise reward
    rand_accumulated_reward(iter)=sum(rand_agent(:,2));
    RL_accumulated_reward(iter)=sum(RL_agent(:,2));
    Bayes_accumulated_reward(iter)=sum(Bayes_agent(:,2));
end
    [val1, pos1]=max(RL_accumulated_reward);
    [val2, pos2]=max(Bayes_accumulated_reward);
    RL_optimal_par=[parameters(pos1,1)];
    Bayes_optimal_par=[parameters(pos2,2)];
    
    %If you change the stochastic value assignment in "environment", the
    %two models change in their respective ability to reach optimality
    disp(['random', 'RL', 'Bayes']);
    disp([mean(rand_accumulated_reward), mean(RL_accumulated_reward), mean(Bayes_accumulated_reward)]);
    