close all
clear all

%loads reward structure
load('Agent_behaviour.mat')
n_choices=2;

for agent=1:length(agent_behav_cells)
    target_behaviour=agent_behav_cells{agent}(:,1);
    target_reward   =agent_behav_cells{agent}(:,2);
    
for iter=1:100
    
    %Montecarlo method
    parameters{agent}(iter,1)=rand(1);          %random search of parameters, in the interval 0<x<1
    parameters{agent}(iter,2)=rand(1)+0.01;     %random search of parameters, in the interval 0.01<x<1.01
    
    %learning rate for RL agent
    alpha=parameters{agent}(iter,1);
    %standard deviation for Bayesian agent
    sigma1 =parameters{agent}(iter,2);

    %temperature for softmax selection function (set as hyper parameter)
    tau  =0.1;
    
    %determine the assumed distribution for the possible events
    values1 = normpdf(1:n_choices,1,sigma1);
    m1=values1(1);
    s1=values1(2);
    %a matrix here is mostly useful in the case of multi-option choices (more than 2) 
    B_probN=[0 1
             1 0]*s1 + eye(n_choices)*m1; %for events that confirm the belief
    B_probB=[0 1
             1 0]*m1 + eye(n_choices)*s1; %for events the conflict with the belief

    RL_errors{agent}(iter)=0;
    Bayes_errors{agent}(iter)=0;    
    for triali=1:length(target_behaviour)
        
        %RL agent
        if triali==1
            Q_values{agent}(triali,:)=[0.5 0.5];
        elseif triali>=2
            %Q value update (Rescorla Wagner): this updates the value
            %associated with the action selected in the previous trial (matching 
            %the actual selection and reward of the targeted agent). The remaining
            %action is reset to zero in this basic version of the algorithm.
            Q_values{agent}(triali,target_behaviour(triali-1))=...
                Q_values{agent}(triali-1,target_behaviour(triali-1)) + alpha*(target_reward(triali-1)-Q_values{agent}(triali-1,target_behaviour(triali-1)));
        end
        %instead of selecting an action based on Q value, the softmax is
        %used to estimate probabilities for both actions: these will be
        %matched with the actual action selections of the targeted agent
        RL_behaviour=softmaxselection(Q_values{agent}(triali,:), tau);
        RL_agent_prob(triali,:)=RL_behaviour{2};

        %Bayesian agent
        if triali==1
            %initial distribution of probabilities
            Bayes_p(triali,:)=ones(1,n_choices)/n_choices;
        elseif triali>=2
            %update of the distribution of probabilities
            if target_behaviour(triali-1)==1
                if target_reward(triali-1)==100 ||  target_reward(triali-1)==-50
                    den=Bayes_p(triali-1,1)*B_probN(1,1)+Bayes_p(triali-1,2)*B_probN(1,2);
                    Bayes_p(triali,:)=(B_probN(target_behaviour(triali-1),:).*Bayes_p(triali-1,:))/den;
                elseif  target_reward(triali-1)==-100 ||  target_reward(triali-1)==50
                    den=Bayes_p(triali-1,1)*B_probN(2,1)+Bayes_p(triali-1,2)*B_probN(2,2);
                    Bayes_p(triali,:)=(B_probB(target_behaviour(triali-1),:).*Bayes_p(triali-1,:))/den;
                end
            elseif target_behaviour(triali-1)==2
                if target_reward(triali-1)==100 ||  target_reward(triali-1)==-50
                    den=Bayes_p(triali-1,1)*B_probN(2,1)+Bayes_p(triali-1,2)*B_probN(2,2);
                    Bayes_p(triali,:)=(B_probN(target_behaviour(triali-1),:).*Bayes_p(triali-1,:))/den;
                elseif  target_reward(triali-1)==-100 ||  target_reward(triali-1)==50
                    den=Bayes_p(triali-1,1)*B_probN(1,1)+Bayes_p(triali-1,2)*B_probN(1,2);
                    Bayes_p(triali,:)=(B_probB(target_behaviour(triali-1),:).*Bayes_p(triali-1,:))/den;
                end
            end
        end

        %it is not necessary to select an action or assign a reward, as we are 
        %trying to match the actual behaviour/reward associations of the targeted agent
        
        %ERROR ESTIMATION can be performed in a vast number of ways. Here we 
        %simply have a "distance" between estimated probability assigned to
        %a choice and actual choice selections.
        RL_errors{agent}(iter)=RL_errors{agent}(iter)+1-RL_agent_prob(triali,target_behaviour(triali));
        Bayes_errors{agent}(iter)=Bayes_errors{agent}(iter)+1-Bayes_p(triali,target_behaviour(triali));

        %if one of the options reaches values close to zero, updates for 
        %that option stall: this below is a simple way to prevent this
        Bayes_p(triali,:)=max(0.1, Bayes_p(triali,:));
    end
    
end
    %Rather than maximise the reward, here we minimise the error. This is just 
    %one example of many possible ways to achieve this objective
    [val1, pos1]=min(RL_errors{agent});
    [val2, pos2]=min(Bayes_errors{agent});
    
    RL_optimal_par(agent,1)= parameters{agent}(pos1,1);
    Bayes_optimal_par(agent,1)= parameters{agent}(pos2,2);
    RL_optimal_par(agent,2)= val1;
    Bayes_optimal_par(agent,2)= val2;

end
    
    disp(RL_optimal_par);
    disp(Bayes_optimal_par)    